# -*- coding: utf-8 -*-
##############################################################################
#
#    wms module for OpenERP, WMS Extended for Open ERP
#    Copyright (C) 2010 SYLEAM (<http://syleam.fr/>)
#              Christophe Chauvet <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms
#
#    wms is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
import netsvc


class StockMoveRemark(osv.osv):
    _name = 'stock.move.remark'
    _description = 'Remark for the stock move'

    _columns = {
        'name': fields.char('Name', size=128, translate=True, required=True),
        'code': fields.char('Code', size=16),
        'redirect': fields.boolean('Redirect', help='Use to made a different treatment, when this remark is selected!'),
    }

    _defaults = {
        'name': lambda *a: False,
        'code': lambda *a: False,
        'redirect': lambda *a: False,
    }

StockMoveRemark()


class StockMove(osv.osv):
    _inherit = 'stock.move'

    _columns = {
        'comment': fields.char('Comment', size=128, help='Enter comment, only if not present in remark list'),
        'remark_id': fields.many2one('stock.move.remark', 'Remark', help='Enter the remark for this product'),
        'purchase_price': fields.float('Last purchase price', help='Store the last purchase price for this product'),
        'average_price': fields.float('Average price', help='Store the weighted average price unit for this product'),
        'invoice_price': fields.float('Invoice price', help='Store the invoice price, check the invoice UOM as well'),
        'invoice_uom_id': fields.many2one('product.uom', 'Invoice UOM', help='Store the invoice UOM'),
        'crossdock_ok': fields.boolean('Crossdock_ok', help='If check, the crossdoc process have been made'),
        'order_qty': fields.float('Order Qty.', help='Usefull when sale module not installed'),
        'move_crossdock_ids': fields.many2many('stock.move', 'stock_move_crossdock_rel', 'receipt_id', 'prepare_id', 'Crossdock movement', ),
    }

    _defaults = {
        'comment': lambda *a: False,
        'remark_id': lambda *a: False,
        'purchase_price': lambda *a: 0.0,
        'average_price': lambda *a: 0.0,
        'invoice_price': lambda *a: 0.0,
        'crossdock_ok': lambda *a: False,
        'order_qty': lambda *a: 0.0,
    }

    def copy(self, cr, uid, id, default=None, context=None):
        """
        Put order Qty from orignal move
        """
        if context is None:
            context = {}
        default['order_qty'] = 0.0

        return super(StockMove, self).copy(cr, uid, id, default, context=context)

    def create(self, cr, uid, values, context=None):
        """
        We must verify if there is a picking_id associate to the move
        and put correctly the warehouse
        if picking is affect to a round, we correct the location_dest_id
        """
        if context is None:
            context = {}

        if values.get('picking_id'):
            pick_obj = self.pool.get('stock.picking')
            pick = pick_obj.browse(cr, uid, values.get('picking_id'), context=context)
            if not pick.warehouse_id and values.get('location_id'):
                loc = self.pool.get('stock.location').read(cr, uid, values.get('location_id'), ['warehouse_id'], context=context)
                if loc and loc['warehouse_id']:
                    pick_obj.write(cr, uid, [values.get('picking_id')], {'warehouse_id': loc['warehouse_id'][0]}, context=context)
            if pick.type == 'out' and pick.round_id:
                values['location_dest_id'] = pick.round_id.default_location_id.id

        return super(StockMove, self).create(cr, uid, values, context=context)

    def write(self, cr, uid, ids, values, context=None):
        """
        If picking is add or replace, recheck the round to apply it on each move line
        """
        if context is None:
            context = {}

        if values.get('picking_id'):
            pick = self.pool.get('stock.picking').browse(cr, uid, values.get('picking_id'), context=context)
            if pick.type == 'out' and pick.round_id:
                values['location_dest_id'] = pick.round_id.default_location_id.id

        return super(StockMove, self).write(cr, uid, ids, values, context=context)

    def action_done(self, cr, uid, ids, context=None):
        """
        This function store average price form product in stock move if configure in comapny
        """
        res = super(StockMove, self).action_done(cr, uid, ids, context)
        company = self.pool.get('res.users').browse(cr, uid, uid, context=context).company_id

        for move in self.browse(cr, uid, ids):
            if (move.state == 'done') \
               and ((move.picking_id.type == 'internal' and company.average_price_stock_picking_internal)\
                    or (move.picking_id.type == 'out' and company.average_price_stock_picking_out)\
                    or (move.picking_id.type == 'delivery' and company.average_price_stock_picking_delivery)):
                self.write(cr, uid, ids, {'average_price': move.product_id.standard_price})
        return res

    def check_assign(self, cr, uid, ids, context=None):
        """
        The assignement must be keep the production_lot and tracking
        and purpose
        """
        print 'check_assign'
        done = []
        count = 0
        pickings = {}
        for move in self.browse(cr, uid, ids):
            if move.product_id.type == 'consu':
                if move.state in ('confirmed', 'waiting'):
                    done.append(move.id)
                pickings[move.picking_id.id] = 1
                continue

            # GR: moves related to internal pickings have nothing to do
            # with the total amount of product.
            if ((not move.picking_id or move.picking_id.type != 'internal')
                 and move.product_id.qty_physical <= 0):
                    continue

            if move.state in ('confirmed', 'waiting'):
                print 'plop'
                prodlot = move.prodlot_id and move.prodlot_id.id or False
                tracking = move.tracking_id and move.tracking_id.id or False
                res = self.pool.get('stock.location')._product_reserve(cr, uid, [move.location_id.id], move.product_id.id, move.product_qty,
                        {'uom': move.product_uom.id, 'prodlot': prodlot, 'tracking': tracking})

                total_qty = move.product_qty
                original_location_id = move.location_id.id

                if res:
                    #_product_available_test depends on the next status for correct functioning
                    #the test does not work correctly if the same product occurs multiple times
                    #in the same order. This is e.g. the case when using the button 'split in two' of
                    #the stock outgoing form
                    r = res.pop(0)
                    self.write(cr, uid, move.id, {'location_id': r[1], 'product_qty': r[0], 'state': 'assigned'})
                    done.append(move.id)
                    pickings[move.picking_id.id] = 1
                    total_qty -= r[0]

                    while res:
                        r = res.pop(0)
                        move_id = self.copy(cr, uid, move.id, {'product_qty': r[0], 'location_id': r[1]})
                        done.append(move_id)
                        total_qty -= r[0]

                    if total_qty > 0:
                        move_id = self.copy(cr, uid, move.id, {'product_qty': total_qty, 'location_id': original_location_id, 'state': 'confirmed'}, context=context)

        if done:
            count += len(done)
            self.write(cr, uid, done, {'state': 'assigned'})

        if count:
            for pick_id in pickings:
                wf_service = netsvc.LocalService("workflow")
                wf_service.trg_write(uid, 'stock.picking', pick_id, cr)
        return count

StockMove()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
