# -*- coding: utf-8 -*-
##############################################################################
#
#    wms module for OpenERP, WMS Extended for Open ERP
#    Copyright (C) 2010 SYLEAM (<http://syleam.fr/>)
#              Christophe Chauvet <christophe.chauvet@syleam.fr>
#
#    This file is a part of wms
#
#    wms is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv
from osv import fields
from tools.translate import _


class Warehouse(osv.osv):
    _inherit = 'stock.warehouse'

    _columns = {
        'company_id': fields.many2one('res.company', 'Company', help='Company where this warehouse is linked'),
        'lot_input_id': fields.many2one('stock.location', 'Location Input', required=False, domain=[('usage', '<>', 'view')]),
        'lot_stock_id': fields.many2one('stock.location', 'Location Stock', required=False, domain=[('usage', '<>', 'view')]),
        'lot_output_id': fields.many2one('stock.location', 'Location Output', required=False, domain=[('usage', '<>', 'view')]),
        'code': fields.char('Code', size=16, help='Enter the warehouse code'),
        'parent_id': fields.many2one('stock.warehouse', 'Parent warehouse', help='Define the parent warehouse'),
        'crossdock_location_id': fields.many2one('stock.location', 'Crossdock location',
            help='Choose the crossdock location on this warehouse'),
        'return_location_id': fields.many2one('stock.location', 'Return location',
            help='Choose the return location on this warehouse'),
        'check_tracking_burst': fields.boolean('check tracking', help='Check tracking on burst out'),
        'keep_missing_product_in': fields.boolean('Keep product in', help='If check, missing product line on receipt order stay with quantity = 0'),
        'force_reserved_location': fields.boolean('Force reserved', help='Force move product in reserved location'),
        'forbid_unsupplied_internal_move': fields.boolean('Forbid unsupplied internal', help="Prevent internal moves with not enough stock level"),
    }

    _defaults = {
        'check_tracking_burst': lambda *a: False,
        'keep_missing_product_in': lambda *a: True,
        'force_reserved_location': lambda *a: False,
        'forbid_unsupplied_internal_move': lambda *a: False,
    }

    def _check_recursion(self, cr, uid, ids):
        """
        Check if the selected parent warehouse create a recursive loop
        """
        level = 100
        while len(ids):
            cr.execute('select distinct parent_id from stock_warehouse where id in %s', (tuple(ids),))
            ids = filter(None, map(lambda x: x[0], cr.fetchall()))
            if not level:
                return False
            level -= 1
        return True

    _constraints = [
        (_check_recursion, 'You cannot create recursive warehouse !', ['parent_id']),
    ]

    def search(self, cr, uid, args, offset=0, limit=None, order=None,
            context=None, count=False):
        """
        Check if default location on the warehouse is correct, if not raise an error
        """
        if context is None:
            context = {}

        ids = super(Warehouse, self).search(cr, uid, args, offset=offset, limit=limit,
                            order=order, context=context, count=count)
        for w in self.browse(cr, uid, ids, context=context):
            if not context.get('new_warehouse', False) and (not w.lot_stock_id or not w.lot_input_id or not w.lot_output_id):
                raise osv.except_osv(_('Error'), _('Warehouse "%s" not correctly configure !') % w.name)

        return ids

Warehouse()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
