# -*- coding: utf-8 -*-
##############################################################################
#
#    wms module for OpenERP, WMS Extended for Open ERP
#    Copyright (C) 2011 SYLEAM Info Services (<http://www.Syleam.fr/>)
#              Sylvain Garancher <sylvain.garancher@syleam.fr>
#
#    This file is a part of wms
#
#    wms is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    wms is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################


import wizard
import pooler
from tools.translate import _

init_form = """<?xml version="1.0" ?>
<form string="%s">
    <field name="product_id" colspan="4"/>
    <field name="quantity"/>
    <field name="remark_id"/>
</form>
""" % (_('Burst'))

init_fields = {
    'product_id': {'string': 'Product', 'type': 'many2one', 'relation': 'product.product', 'required': True, },
    'quantity': {'string': 'Quantity', 'type': 'float', 'required': True, },
    'remark_id': {'string': 'Observation', 'type': 'many2one', 'relation': 'stock.move.remark', },
}


def _init(self, cr, uid, data, context):
    if context is None:
        context = {}

    pool = pooler.get_pool(cr.dbname)
    burst = pool.get('stock.burst').browse(cr, uid, data['id'], context=context)
    pick = pool.get('stock.picking').browse(cr, uid, burst.stock_picking_id.id, context=context)
    if pick.state != 'assigned' and burst.type == 'out':
        raise wizard.except_wizard(_('Error'), _('Picking is not assigned!'))

    # Set default values
    data['form']['product_id'] = False
    data['form']['quantity'] = 1.
    data['form']['remark_id'] = False

    return data['form']


def _save(self, cr, uid, data, context):
    if context is None:
        context = {}

    pool = pooler.get_pool(cr.dbname)
    pick = pool.get('stock.burst').browse(cr, uid, data['id'], context=context)

    if pick.stock_picking_id.type == 'out':
        pro_ok = False
        pro_qty = 0.0
        for l in pick.stock_picking_id.move_lines:
            if l.product_id.id == data['form'].get('product_id', False):
                pro_ok = True
                pro_qty += l.product_qty

        for l in pick.line_ids:
            if l.product_id.id == data['form'].get('product_id', False):
                pro_qty -= l.quantity

        if not pro_ok:
            raise wizard.except_wizard(_('Error'), _('This product is not on the original picking'))

        if data['form'].get('quantity', 0.) > pro_qty:
            raise wizard.except_wizard(_('Error'), _('This quantity "%.2f" is greater than the missing quantity "%.2f"') % (data['form'].get('quantity', 0.), pro_qty))

    # Create line
    stock_burst_line_obj = pool.get('stock.burst.line')
    vals = {
        'burst_id': (data['id'] or data['ids'][0]),
        'product_id': data['form'].get('product_id', False),
        'quantity': data['form'].get('quantity', 0.),
        'remark_id': data['form'].get('remark_id', False),
    }
    stock_burst_line_obj.create(cr, uid, vals, context=context)

    return {}


class wiz_burst(wizard.interface):

    states = {
        'init': {
            'actions': [_init],
            'result': {
                'type': 'form',
                'arch': init_form,
                'fields': init_fields,
                'state': [('end', 'Finnish', 'gtk-cancel'), ('valid', 'OK', 'gtk-ok', True)],
            }
        },
        'valid': {
            'actions': [_save],
            'result': {
                'type': 'state',
                'state': 'init',
            }
        }
    }

wiz_burst('wms.burst')

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
